<?php

/**
 * @file
 * Administrations functions for the dbee module, on the admin/settings/dbee
 * page.
 */

/**
 * Form builder; administer dbee module options.
 *
 * Add on option to enable to loggin using either username or email address.
 *
 * @see dbee_admin_form_validate()
 * @see dbee_admin_form_submit()
 * @ingroup forms
 */
function dbee_admin_form(&$form, &$form_state) {

  form_load_include($form_state, 'inc', 'dbee', 'includes/dbee.admin');

  $dbee_custom_search = (module_exists('search') || variable_get('dbee_custom_search', 0));

  $form['dbee'] = array(
    '#type' => 'fieldset',
    '#title' => t('About performance'),
    '#description' => t('Read this if you meet some <strong>performance issues</strong>.'),
    '#group' => 'settings',
    '#weight' => -1,
  );

  $perf_case = '<p>' . t('You may meet performance issues, due to the %module module, <strong>on one specific use case</strong> matching the both conditions below :', array('%module' => t('DataBase email Encryption'))) . '</p>';
  $conditions = array();
  $conditions[] = t('your site contains a huge number of registered users,');
  $conditions[] = t('and it is running on slow server.');
  $perf_case .= theme('item_list', array('items' => $conditions));

  $perf_case .= '<p>' . t('Thoses performance issues <strong>may happen on</strong> :') . '</p>';


  $conditions = array();
  $conditions[] = t('editing User informations from the UI (on the user/%/edit page),');
  if ($dbee_custom_search) {
    $conditions[] =  t('searching a user (on the <a href="@url">search/user</a> page),', array('@url' => url('search/user')));
  }
  if (module_exists('email_registration') || module_exists('logintoboggan')) {
    $modules_args = array(
      '%module1' => 'Email Registration',
      '!url1' => 'https://www.drupal.org/project/email_registration',
      '%module2' => 'LoginToboggan',
      '!url2' => 'https://www.drupal.org/project/logintoboggan',
    );
    $conditions[] =  t('logging in using email, with one of the <a href="!url1">%module1</a> or <a href="!url2">%module2</a> contrib modules,', $modules_args);
  }

  $perf_case .= theme('item_list', array('items' => $conditions));

  $perf_case .= '<p>' . t('<strong>In order to solve them</strong>, keeping the dbee module :') . '</p>';
  $conditions = array();
  $conditions[] = t('Set the encryption method to "Basic".') . '<br />' .
    t('<strong>Note that</strong> your emails will still be obfuscated but could be very easily cracked.');
  if ($dbee_custom_search) {
    $conditions[] = t('And for solving performance on searching users : check the "disabling searching users via uncompleted email address" option.') . '<br />' .
      t("<strong>Note that</strong>, as a limitation, only completed emails will be finded. if this checkbox is checked, on site storing a user with the 'john@example.com' mail value, searching this user using the key word 'hn@examp' will fail but the whole email address search key 'john@example.com' will succeed. In both cases, the result will be returned very fast.");
  }
  $perf_case .= theme('item_list', array('items' => $conditions));

  $form['dbee']['performance'] = array(
    '#markup' => $perf_case, //t('You may meet performance issue, due to the dbee module, on a specific case : your site contain with a huge number of registered users running on slow server : it will be solved using the "Basic" encryption method. Note that your datas will still be offusced but could be easily cracked.'),
  );

  if ($dbee_custom_search) {
    $form['dbee']['custom_search'] = array(
      '#type' => 'checkbox',
      '#title' => t('Disable searching users via uncompleted email address.'),
      '#default_value' => variable_get('dbee_custom_search', 0),
      '#description' => t('Read above for explanation on this option.'),
    );
  }

  $form['#validate'][] = 'dbee_admin_form_validate';
  $form['actions']['submit']['#submit'][] = 'dbee_admin_form_submit';

  // Remove delete button.
  if (isset($form['actions']['delete'])) {
    $form['actions']['delete']['#access'] = FALSE;
  }

  // Re-encrypt all users.
  $form['dbee_reencrypt'] = array(
    '#type' => 'checkbox',
    '#title' => '<strong>' . t('Re-encrypted all users') . '</strong>',
    '#description' => t('If checked, all users will be re-encrypted.'),
    '#default_value' => 1,
    '#weight' => 45,
  );

  // Disable the edition of the name and description.
  $form['label']['#disabled'] = TRUE;
  $form['description']['#disabled'] = TRUE;
  $form['description']['#type'] = 'textfield';
  $form['name']['#disabled'] = TRUE;

  // Can not be disabled.
  if (!empty($form['general_settings']['enabled']['#default_value'])) {
    $form['general_settings']['enabled']['#disabled'] = TRUE;
  }

  // Always provide the default method
  if (empty($form['method_settings']['encrypt_encryption_method']['#options']['default'])) {
    $methods = encrypt_get_encryption_methods();
    $form['method_settings']['encrypt_encryption_method']['#options']['default'] = $methods['default']['title'] . ' <span class="encrypt-warning">' . t('(Deprecated)') . '</span>';
  }
}

/**
 * Validate dbee module options.
 */
function dbee_admin_form_validate($form, &$form_state) {
  if (!user_access('administer dbee')) {
    form_set_error('', t('You are not allowed to change options.'));
  }
  if (!form_get_errors()) {
    $config_name = $form_state['values']['name'];
    $prev_config = encrypt_get_config($config_name);
    $prev_method = (!empty($prev_config['method'])) ? $prev_config['method'] : FALSE;
    $new_method = (!empty($form_state['values']['encrypt_encryption_method'])) ? (string) $form_state['values']['encrypt_encryption_method'] : FALSE;
    if ($new_method && $new_method != $prev_method) {
      // Method has changed.
      foreach (array('default', 'none') as $fixed_method) {
        if ($new_method == $fixed_method || $prev_method == $fixed_method) {
          // Force reencrypting all users. // @TODO : confirm form ?
          // Because the dbee module act differently with this method.
          if (empty($form_state['values']['dbee_reencrypt'])) {
            $form_state['values']['dbee_reencrypt'] = TRUE;
          }
        }
      }
    }
  }
}

/**
 * Submit dbee module options.
 */
function dbee_admin_form_submit($form, &$form_state) {

  if ($form_state['values']['custom_search'] != variable_get('dbee_custom_search', 0)) {
    variable_set('dbee_custom_search', trim(check_plain($form_state['values']['custom_search'])));
    drupal_set_message(t('New setting has been saved'));
  }

  // Re-encrypt all existing email addresses.
  if (!empty($form_state['values']['dbee_reencrypt'])) {
    module_load_include('inc', 'dbee', 'includes/dbee.users');
    dbee_update_crypt_all('change');
  }
}

/**
 * Display a table will all encryption configurations.
 *
 * Alter the data to prevent deleting the dbee configuration.
 */
function dbee_encrypt_configs_list() {
  module_load_include('inc', 'encrypt', 'includes/encrypt.admin');
  $build = encrypt_configs_list();
  if (!empty($build['encrypt_configs_list_table']['#rows'])) {
    $search = array(
      url(ENCRYPT_MENU_PATH . '/delete/dbee'),
      url(ENCRYPT_MENU_PATH . '/default/dbee'),
    );
    foreach ($build['encrypt_configs_list_table']['#rows'] as $row_id => $cols) {
      foreach ($cols as $col_id => $cell) {
        foreach ($search as $url) {
          if (!empty($cell['data']) && strpos($cell['data'], $url) !== FALSE) {
            // Link for deleting or make default for dbee config : delete the cell.
            $build['encrypt_configs_list_table']['#rows'][$row_id][$col_id]['data'] = '';
          }
        }
      }
    }
  }
  return $build;
}
